@@ -35,46 +35,77 @@
 #include "polarssl/ssl_cache.h"
 #include "polarssl/error.h"
 
+#if POLARSSL_VERSION_NUMBER < 0x01030000
+#define x509_crt_parse_file x509parse_crtfile
+#define pk_parse_keyfile x509parse_keyfile
+#define x509_crl_parse_file x509parse_crlfile
+#define x509_dn_gets x509parse_dn_gets
+#define x509_serial_gets x509parse_serial_gets
+#endif
+
 typedef struct type_sni_list {
 	t_charlist *hostname;
-	rsa_context *private_key;
-	x509_cert *certificate;
-	x509_cert *ca_certificate;
-	x509_crl  *ca_crl;
+	pk_context *private_key;
+	x509_crt *certificate;
+	x509_crt *ca_certificate;
+	x509_crl *ca_crl;
 
 	struct type_sni_list *next;
 } t_sni_list;
 
 static int ciphersuites[] = {
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	TLS_RSA_WITH_RC4_128_SHA,
-	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA256,
-	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA,
-	TLS_DHE_RSA_WITH_AES_256_CBC_SHA256,
+#else
+	TLS_ECDHE_RSA_WITH_RC4_128_SHA,
+	TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
+	TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
+#endif
 	TLS_DHE_RSA_WITH_AES_256_CBC_SHA,
-	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA256,
-	TLS_RSA_WITH_AES_256_CBC_SHA256,
-	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA,
+	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA,
+#if POLARSSL_VERSION_NUMBER >= 0x01030000
+	TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
+	TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
+#endif
+	TLS_DHE_RSA_WITH_AES_128_CBC_SHA,
+	TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA,
 	TLS_RSA_WITH_AES_256_CBC_SHA,
+	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA,
+	TLS_RSA_WITH_AES_128_CBC_SHA,
+	TLS_RSA_WITH_CAMELLIA_128_CBC_SHA,
 	0
 };
+
 #if POLARSSL_VERSION_NUMBER >= 0x01020700
 static int ciphersuites_tls12[] = {
+#if POLARSSL_VERSION_NUMBER >= 0x01030000
+	TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
+	TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
+	TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384,
+	TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
+	TLS_ECDHE_ECDSA_WITH_CAMELLIA_256_CBC_SHA384,
+	TLS_ECDHE_RSA_WITH_CAMELLIA_256_CBC_SHA384,
+	TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384,
+	TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
+#endif
 	TLS_DHE_RSA_WITH_AES_256_GCM_SHA384,
-	TLS_DHE_RSA_WITH_AES_128_GCM_SHA256,
-	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA256,
-	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA,
 	TLS_DHE_RSA_WITH_AES_256_CBC_SHA256,
+	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA256,
 	TLS_DHE_RSA_WITH_AES_256_CBC_SHA,
+	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA,
 	TLS_RSA_WITH_AES_256_GCM_SHA384,
-	TLS_RSA_WITH_AES_128_GCM_SHA256,
-	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA256,
 	TLS_RSA_WITH_AES_256_CBC_SHA256,
-	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA,
+	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA256,
 	TLS_RSA_WITH_AES_256_CBC_SHA,
+	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA,
 	0
 };
 #endif
 
+#if POLARSSL_VERSION_NUMBER == 0x01020900
+static pthread_mutex_t handshake_mutex;
+#endif
+
 static char *dhm_4096_P =
 	"FFFFFFFFFFFFFFFFC90FDAA22168C234C4C6628B80DC1CD1"
 	"29024E088A67CC74020BBEA63B139B22514A08798E3404DD"
@@ -101,7 +132,9 @@ static char *dhm_4096_P =
 static char *dhm_4096_G = "02";
 
 static char *ssl_error_logfile;
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 static rsa_context rsa;
+#endif
 static pthread_mutex_t random_mutex;
 static pthread_mutex_t cache_mutex;
 static ssl_cache_context cache;
@@ -114,10 +147,14 @@ static entropy_context entropy;
 int init_ssl_module(char *logfile) {
 	ssl_error_logfile = logfile;
 
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	rsa_init(&rsa, RSA_PKCS_V15, 0);
+#endif
 
 	entropy_init(&entropy);
-	ctr_drbg_init(&ctr_drbg, entropy_func, &entropy, (unsigned char*)"Hiawatha_RND", 10);
+	if (ctr_drbg_init(&ctr_drbg, entropy_func, &entropy, (unsigned char*)"Hiawatha_RND", 10) != 0) {
+		return -1;
+	}
 	ctr_drbg_set_prediction_resistance(&ctr_drbg, CTR_DRBG_PR_OFF);
 
 	ssl_cache_init(&cache);
@@ -127,6 +164,10 @@ int init_ssl_module(char *logfile) {
 		return -1;
 	} else if (pthread_mutex_init(&cache_mutex, NULL) != 0) {
 		return -1;
+#if POLARSSL_VERSION_NUMBER == 0x01020900
+	} else if (pthread_mutex_init(&handshake_mutex, NULL) != 0) {
+		return -1;
+#endif
 	}
 
 	return 0;
@@ -134,8 +175,8 @@ int init_ssl_module(char *logfile) {
 
 /* Add SNI information to list
  */
-int ssl_register_sni(t_charlist *hostname, rsa_context *private_key, x509_cert *certificate,
-                x509_cert *ca_certificate, x509_crl *ca_crl) {
+int ssl_register_sni(t_charlist *hostname, pk_context *private_key, x509_crt *certificate,
+                x509_crt *ca_certificate, x509_crl *ca_crl) {
 	t_sni_list *sni;
 
 	if ((sni = (t_sni_list*)malloc(sizeof(t_sni_list))) == NULL) {
@@ -157,12 +198,12 @@ int ssl_register_sni(t_charlist *hostname, rsa_context *private_key, x509_cert *
 /* SSL debug callback function
  */
 #ifdef ENABLE_DEBUG
-static void ssl_debug(void UNUSED(*ctx), int level, const char *str) {
+static void ssl_debug(void *thread_id, int level, const char *str) {
 	if (level >= SSL_DEBUG_LEVEL) {
 		return;
 	}
 
-	log_string(ssl_error_logfile, "PolarSSL:%s", str);
+	log_string(ssl_error_logfile, "PolarSSL (%d):%s", *(int*)thread_id, str);
 }
 #endif
 
@@ -180,6 +221,7 @@ static int ssl_random(void *p_rng, unsigned char *output, size_t len) {
 
 /* Required to use the SSL cache in a multithreaded application
  */
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 static int ssl_get_cache(void *data, ssl_session *session) {
 	int result;
 
@@ -199,6 +241,7 @@ static int ssl_set_cache(void *data, const ssl_session *session) {
 
 	return result;
 }
+#endif
 
 static void print_ssl_error(char *message, int code) {
 	char cause[1024];
@@ -211,29 +254,37 @@ static void print_ssl_error(char *message, int code) {
 
 /* Load private key and certificate from file
  */
-int ssl_load_key_cert(char *file, rsa_context **private_key, x509_cert **certificate) {
+int ssl_load_key_cert(char *file, pk_context **private_key, x509_crt **certificate) {
 	int result;
 
 	if (file == NULL) {
 		return -1;
 	}
 
-	if ((*private_key = (rsa_context*)malloc(sizeof(rsa_context))) == NULL) {
+	if ((*private_key = (pk_context*)malloc(sizeof(pk_context))) == NULL) {
 		return -1;
 	}
-	memset(*private_key, 0, sizeof(rsa_context));
+#if POLARSSL_VERSION_NUMBER >= 0x01030000
+	pk_init(*private_key);
+#else
+	memset(*private_key, 0, sizeof(pk_context));
+#endif
 
-	if ((result = x509parse_keyfile(*private_key, file, NULL)) != 0) {
+	if ((result = pk_parse_keyfile(*private_key, file, NULL)) != 0) {
 		print_ssl_error("Error loading RSA private key", result);
 		return -1;
 	}
 
-	if ((*certificate = (x509_cert*)malloc(sizeof(x509_cert))) == NULL) {
+	if ((*certificate = (x509_crt*)malloc(sizeof(x509_crt))) == NULL) {
 		return -1;
 	}
-	memset(*certificate, 0, sizeof(x509_cert));
+#if POLARSSL_VERSION_NUMBER >= 0x01030000
+	x509_crt_init(*certificate);
+#else
+	memset(*certificate, 0, sizeof(x509_crt));
+#endif
 
-	if ((result = x509parse_crtfile(*certificate, file)) != 0) {
+	if ((result = x509_crt_parse_file(*certificate, file)) != 0) {
 		print_ssl_error("Error loading X.509 certificates", result);
 		return -1;
 	}
@@ -243,19 +294,23 @@ int ssl_load_key_cert(char *file, rsa_context **private_key, x509_cert **certifi
 
 /* Load CA certificate from file.
  */
-int ssl_load_ca_cert(char *file, x509_cert **ca_certificate) {
+int ssl_load_ca_cert(char *file, x509_crt **ca_certificate) {
 	int result;
 
 	if (file == NULL) {
 		return -1;
 	}
 
-	if ((*ca_certificate = (x509_cert*)malloc(sizeof(x509_cert))) == NULL) {
+	if ((*ca_certificate = (x509_crt*)malloc(sizeof(x509_crt))) == NULL) {
 		return -1;
 	}
-	memset(*ca_certificate, 0, sizeof(x509_cert));
+#if POLARSSL_VERSION_NUMBER >= 0x01030000
+	x509_crt_init(*ca_certificate);
+#else
+	memset(*ca_certificate, 0, sizeof(x509_crt));
+#endif
 
-	if ((result = x509parse_crtfile(*ca_certificate, file)) != 0) {
+	if ((result = x509_crt_parse_file(*ca_certificate, file)) != 0) {
 		print_ssl_error("Error loading X.509 CA certificate", result);
 		return -1;
 	}
@@ -275,9 +330,13 @@ int ssl_load_ca_crl(char *file, x509_crl **ca_crl) {
 	if ((*ca_crl = (x509_crl*)malloc(sizeof(x509_crl))) == NULL) {
 		return -1;
 	}
+#if POLARSSL_VERSION_NUMBER >= 0x01030000
+	x509_crl_init(*ca_crl);
+#else
 	memset(*ca_crl, 0, sizeof(x509_crl));
+#endif
 
-	if ((result = x509parse_crlfile(*ca_crl, file)) != 0) {
+	if ((result = x509_crl_parse_file(*ca_crl, file)) != 0) {
 		print_ssl_error("Error loading X.509 CA CRL", result);
 		return -1;
 	}
@@ -351,11 +410,15 @@ int ssl_accept(t_ssl_accept_data *sad) {
 	ssl_set_renegotiation(sad->context, SSL_RENEGOTIATION_DISABLED);
 	ssl_set_rng(sad->context, ssl_random, &ctr_drbg);
 #ifdef ENABLE_DEBUG
-	ssl_set_dbg(sad->context, ssl_debug, stderr);
+	ssl_set_dbg(sad->context, ssl_debug, &(sad->thread_id));
 #endif
 	ssl_set_bio(sad->context, net_recv, sad->client_fd, net_send, sad->client_fd);
 	ssl_set_sni(sad->context, sni_callback, sad);
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	ssl_set_session_cache(sad->context, ssl_get_cache, &cache, ssl_set_cache, &cache);
+#else
+	ssl_set_session_cache(sad->context, ssl_cache_get, &cache, ssl_cache_set, &cache);
+#endif
 
 #if POLARSSL_VERSION_NUMBER >= 0x01020700
 	ssl_set_ciphersuites_for_version(sad->context, ciphersuites, SSL_MAJOR_VERSION_3, SSL_MINOR_VERSION_0);
@@ -384,6 +447,9 @@ int ssl_accept(t_ssl_accept_data *sad) {
 	setsockopt(*(sad->client_fd), SOL_SOCKET, SO_RCVTIMEO, (void*)&timer, sizeof(struct timeval));
 	start_time = time(NULL);
 
+#if POLARSSL_VERSION_NUMBER == 0x01020900
+	pthread_mutex_lock(&handshake_mutex);
+#endif
 	result = 0;
 	while ((handshake = ssl_handshake(sad->context)) != 0) {
 		if ((handshake != POLARSSL_ERR_NET_WANT_READ) && (handshake != POLARSSL_ERR_NET_WANT_WRITE)) {
@@ -399,10 +465,15 @@ int ssl_accept(t_ssl_accept_data *sad) {
 			break;
 		}
 	}
+#if POLARSSL_VERSION_NUMBER == 0x01020900
+	pthread_mutex_unlock(&handshake_mutex);
+#endif
 
-	timer.tv_sec = 0;
-	timer.tv_usec = 0;
-	setsockopt(*(sad->client_fd), SOL_SOCKET, SO_RCVTIMEO, (void*)&timer, sizeof(struct timeval));
+	if (result == 0) {
+		timer.tv_sec = 0;
+		timer.tv_usec = 0;
+		setsockopt(*(sad->client_fd), SOL_SOCKET, SO_RCVTIMEO, (void*)&timer, sizeof(struct timeval));
+	}
 
 	return result;
 }
@@ -448,39 +519,35 @@ int ssl_send(ssl_context *ssl, const char *buffer, unsigned int length) {
 /* Check if peer sent a client certificate
  */
 bool ssl_has_peer_cert(ssl_context *context) {
-	if (context->session == NULL) {
-		return false;
-	} else if (context->session->peer_cert == NULL) {
-		return false;
-	}
-
-	return true;
+	return ssl_get_peer_cert(context) != NULL;
 }
 
 /* Get information from peer certificate
  */
 int get_peer_cert_info(ssl_context *context, char *subject_dn, char *issuer_dn, char *serial_nr, int length) {
-	if (ssl_has_peer_cert(context) == false) {
+	const x509_crt *peer_cert;
+
+	if ((peer_cert = ssl_get_peer_cert(context)) == NULL) {
 		return -1;
 	}
 
 	/* Subject DN
 	 */
-	if (x509parse_dn_gets(subject_dn, length, &(context->session->peer_cert->subject)) == -1) {
+	if (x509_dn_gets(subject_dn, length, &(peer_cert->subject)) == -1) {
 		return -1;
 	}
 	subject_dn[length - 1] = '\0';
 
 	/* Issuer DN
 	 */
-	if (x509parse_dn_gets(issuer_dn, length, &(context->session->peer_cert->issuer)) == -1) {
+	if (x509_dn_gets(issuer_dn, length, &(peer_cert->issuer)) == -1) {
 		return -1;
 	}
 	issuer_dn[length - 1] = '\0';
 
 	/* Serial number
 	 */
-	if (x509parse_serial_gets(serial_nr, length, &(context->session->peer_cert->serial)) == -1) {
+	if (x509_serial_gets(serial_nr, length, &(peer_cert->serial)) == -1) {
 		return -1;
 	}
 	serial_nr[length - 1] = '\0';
@@ -506,12 +573,18 @@ void ssl_close(ssl_context *ssl) {
 /* Clean up SSL library
  */
 void ssl_shutdown(void) {
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	rsa_free(&rsa);
+#endif
 	ssl_cache_free(&cache);
 }
 
 #ifdef ENABLE_RPROXY
 int ssl_connect(ssl_context *ssl, int *sock, char *hostname) {
+#ifdef ENABLE_DEBUG
+	int no_thread_id = 0;
+#endif
+
 	memset(ssl, 0, sizeof(ssl_context));
 	if (ssl_init(ssl) != 0) {
 		return -1;
@@ -522,14 +595,14 @@ int ssl_connect(ssl_context *ssl, int *sock, char *hostname) {
 
 	ssl_set_rng(ssl, ssl_random, &ctr_drbg);
 #ifdef ENABLE_DEBUG
-	ssl_set_dbg(ssl, ssl_debug, stderr);
+	ssl_set_dbg(ssl, ssl_debug, &no_thread_id);
 #endif
 	ssl_set_bio(ssl, net_recv, sock, net_send, sock);
 
 	if (hostname != NULL) {
 		ssl_set_hostname(ssl, hostname);
 	}
-	ssl_set_ciphersuites(ssl, ciphersuites + 1);
+	ssl_set_ciphersuites(ssl, ciphersuites + (ciphersuites[0] == TLS_RSA_WITH_RC4_128_SHA ? 1 : 0));
 
 	if (ssl_handshake(ssl) != 0) {
 		return -1;
