@@ -67,6 +67,7 @@ extern char *fb_filesystem;
 extern char *fb_symlink;
 extern char *fb_alterlist;
 extern char *hs_eol;
+extern char *hs_conn;
 extern char *hs_conlen;
 extern char *hs_contyp;
 extern char *hs_forwarded;
@@ -79,7 +80,7 @@ int send_file(t_session *session) {
 	off_t file_size, send_begin, send_end, send_size;
 	int  retval, handle = -1;
 	struct stat status;
-	struct tm *fdate;
+	struct tm fdate;
 #ifdef ENABLE_CACHE
 	t_cached_object *cached_object;
 #endif
@@ -198,6 +199,13 @@ int send_file(t_session *session) {
 	 */
 	if (session->handling_error == false) {
 		if ((range = get_http_header("Range:", session->http_headers)) != NULL) {
+			/* Check for multi-range
+			 */
+			if (strchr(range, ',') != NULL) {
+				close(handle);
+				return 416;
+			}
+
 			if (strncmp(range, "bytes=", 6) == 0) {
 				if ((range = strdup(range + 6)) == NULL) {
 					close(handle);
@@ -285,11 +293,11 @@ int send_file(t_session *session) {
 		 */
 		if (fstat(handle, &status) == -1) {
 			break;
-		} else if ((fdate = gmtime(&(status.st_mtime))) == NULL) {
+		} else if (gmtime_r(&(status.st_mtime), &fdate) == NULL) {
 			break;
 		} else if (send_buffer(session, "Last-Modified: ", 15) == -1) {
 			break;
-		} else if (strftime(value, VALUE_SIZE, "%a, %d %b %Y %X GMT\r\n", fdate) == 0) {
+		} else if (strftime(value, VALUE_SIZE, "%a, %d %b %Y %X GMT\r\n", &fdate) == 0) {
 			break;
 		} else if (send_buffer(session, value, strlen(value)) == -1) {
 			break;
@@ -482,7 +490,7 @@ int execute_cgi(t_session *session) {
 	if ((wrap_cgi == false) && (session->cgi_type != fastcgi)) {
 		check_file_exists = true;
 	} else if ((session->cgi_type == fastcgi) && (session->fcgi_server != NULL)) {
-		check_file_exists = session->fcgi_server->localhost;
+		check_file_exists = false;
 	} else {
 		check_file_exists = false;
 	}
@@ -1325,9 +1333,10 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 	t_rproxy_options options;
 	t_rproxy_webserver webserver;
 	t_rproxy_result rproxy_result;
-	char buffer[RPROXY_BUFFER_SIZE + 1], *end_of_header;
+	char buffer[RPROXY_BUFFER_SIZE + 1], *end_of_header, *str, *eol;
 	int bytes_read, bytes_in_buffer = 0, result = 200, code, poll_result;
-	bool header_read = false, keep_reading = true;
+	int can_read, content_length = -1, content_read = 0;
+	bool header_read = false, keep_reading = true, keep_alive, chunked = false;
 	struct pollfd poll_data;
 	time_t deadline;
 #ifdef ENABLE_CACHE
@@ -1340,6 +1349,8 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 	session->current_task = "proxy request";
 #endif
 
+	keep_alive = session->keep_alive;
+
 #ifdef ENABLE_CACHE
 	/* Search for CGI output in cache
 	 */
@@ -1368,23 +1379,58 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 #endif
 	init_rproxy_result(&rproxy_result);
 
-	session->keep_alive = false;
-
-	/* Connect to webserver
-	 */
-	if ((webserver.socket = connect_to_webserver(rproxy)) == -1) {
-		return 503;
+	if (session->rproxy_kept_alive && ((same_ip(&(session->rproxy_addr), &(rproxy->ip_addr)) == false) || (session->rproxy_port != rproxy->port))) {
+#ifdef ENABLE_SSL
+		if (session->rproxy_use_ssl) {
+			ssl_close(&(session->rproxy_ssl));
+		}
+#endif
+		close(session->rproxy_socket);
+		session->rproxy_kept_alive = false;
 	}
 
+	/* Test if kept-alive connection is still alive
+	 */
+	if (session->rproxy_kept_alive) {
+		if (recv(session->rproxy_socket, buffer, 1, MSG_DONTWAIT | MSG_PEEK) == -1) {
 #ifdef ENABLE_SSL
-	webserver.use_ssl = rproxy->use_ssl;
-	if (webserver.use_ssl) {
-		if (ssl_connect(&(webserver.ssl), &(webserver.socket), rproxy->hostname) == -1) {
-			close(webserver.socket);
-			return 503;
+			if (session->rproxy_use_ssl) {
+				ssl_close(&(session->rproxy_ssl));
+			}
+#endif
+			close(session->rproxy_socket);
+
+			session->rproxy_kept_alive = false;
 		}
 	}
+
+	if (session->rproxy_kept_alive) {
+		/* Use kept alive connection
+		 */
+		webserver.socket = session->rproxy_socket;
+#ifdef ENABLE_SSL
+		webserver.use_ssl = session->rproxy_use_ssl;
+		if (webserver.use_ssl) {
+			memcpy(&(webserver.ssl), &(session->rproxy_ssl), sizeof(ssl_context));
+		}
 #endif
+	} else {
+		/* Connect to webserver
+		 */
+		if ((webserver.socket = connect_to_server(&(rproxy->ip_addr), rproxy->port)) == -1) {
+			return 503;
+		}
+
+#ifdef ENABLE_SSL
+		webserver.use_ssl = rproxy->use_ssl;
+		if (webserver.use_ssl) {
+			if (ssl_connect(&(webserver.ssl), &(webserver.socket), rproxy->hostname) == -1) {
+				close(webserver.socket);
+				return 503;
+			}
+		}
+#endif
+	}
 
 	/* Send request to webserver
 	 */
@@ -1415,6 +1461,7 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 				if (errno != EINTR) {
 					result = -1;
 					keep_reading = false;
+					keep_alive = false;
 				}
 				break;
 			case 0:
@@ -1424,6 +1471,15 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 				}
 				break;
 			default:
+				if (content_length == -1) {
+					can_read = RPROXY_BUFFER_SIZE - bytes_in_buffer;
+				} else {
+					can_read = content_length - content_read;
+					if (can_read > RPROXY_BUFFER_SIZE) {
+						can_read = RPROXY_BUFFER_SIZE;
+					}
+				}
+
 #ifdef ENABLE_SSL
 			    if (webserver.use_ssl) {
 					bytes_read = ssl_receive(&(webserver.ssl), buffer + bytes_in_buffer, RPROXY_BUFFER_SIZE - bytes_in_buffer);
@@ -1436,6 +1492,7 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 						if (errno != EINTR) {
 							result = -1;
 							keep_reading = false;
+							keep_alive = false;
 						}
 						break;
 					case 0:
@@ -1449,7 +1506,7 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 
 							*(buffer + bytes_in_buffer) = '\0';
 							if ((end_of_header = strstr(buffer, "\r\n\r\n")) != NULL) {
-								*end_of_header = '\0';
+								*(end_of_header + 2) = '\0';
 								if ((code = extract_http_code(buffer)) != -1) {
 									session->return_code = code;
 								}
@@ -1457,6 +1514,7 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 								if ((code != 200) && (session->host->trigger_on_cgi_status)) {
 									result = code;
 									keep_reading = false;
+									keep_alive = false;
 									*end_of_header = '\r';
 									break;
 								}
@@ -1472,7 +1530,45 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 
 								handle_remove_header_for_rproxy_cache(session, buffer);
 #endif
-								*end_of_header = '\r';
+
+								/* Parse content length
+								 */
+								if ((str = strstr(buffer, hs_conlen)) != NULL) {
+									str += strlen(hs_conlen);
+									if ((eol = strchr(str, '\r')) != NULL) {
+										*eol = '\0';
+										content_length = str2int(str);
+										*eol = '\r';
+									}
+								}
+
+								/* Determine if is chunked encoding
+								 */
+								if (strstr(buffer, hs_chunked) != NULL) {
+									chunked = true;
+								}
+
+								/* Determine connection type
+								 */
+								if (strstr(buffer, " HTTP/1.0\r\n") != NULL) {
+									keep_alive = false;
+								} else if (strstr(buffer, " HTTP/1.1\r\n") != NULL) {
+									keep_alive = true;
+									if ((str = strstr(buffer, hs_conn)) != NULL) {
+										str += strlen(hs_conn);
+										if ((eol = strchr(str, '\r')) != NULL) {
+											*eol = '\0';
+											if (strcmp(str, "close") == 0) {
+												keep_alive = false;
+											}
+											*eol = '\r';
+										}
+									}
+								}
+
+								content_read = bytes_in_buffer - (end_of_header + 4 - buffer);
+
+								*(end_of_header + 2)= '\r';
 
 								bytes_read = bytes_in_buffer;
 								bytes_in_buffer = 0;
@@ -1480,10 +1576,24 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 							} else if (bytes_in_buffer == RPROXY_BUFFER_SIZE) {
 								result = -1;
 								keep_reading = false;
+								keep_alive = false;
 								break;
 							} else {
 								continue;
 							}
+						} else {
+							content_read += bytes_read;
+							*(buffer + bytes_read) = '\0';
+						}
+
+						if (content_read == content_length) {
+							keep_reading = false;
+						}
+
+						if (chunked && (bytes_read >= 7)) {
+							if (memcmp(buffer + bytes_read - 7, "\r\n0\r\n\r\n", 7) == 0) {
+								keep_reading = false;
+							}
 						}
 
 						/* Send data to client
@@ -1491,6 +1601,7 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 						if (send_buffer(session, buffer, bytes_read) == -1) {
 							result = -1;
 							keep_reading = false;
+							keep_alive = false;
 							break;
 						}
 
@@ -1519,14 +1630,32 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 	}
 #endif
 
-	/* Close connection to webserver
-	 */
+	if (keep_alive == false) {
+		/* Close connection to webserver
+		 */
 #ifdef ENABLE_SSL
-	if (webserver.use_ssl) {
-		ssl_close(&(webserver.ssl));
-	}
+		if (webserver.use_ssl) {
+			ssl_close(&(webserver.ssl));
+		}
 #endif
-	close(webserver.socket);
+		close(webserver.socket);
+
+		session->keep_alive = false;
+		session->rproxy_kept_alive = false;
+	} else if (session->rproxy_kept_alive == false) {
+		/* Keep connection alive
+		 */
+		memcpy(&(session->rproxy_addr), &(rproxy->ip_addr), sizeof(t_ip_addr));
+		session->rproxy_port = rproxy->port;
+		session->rproxy_socket = webserver.socket;
+#ifdef ENABLE_SSL
+		session->rproxy_use_ssl = webserver.use_ssl;
+		if (session->rproxy_use_ssl) {
+			memcpy(&(session->rproxy_ssl), &(webserver.ssl), sizeof(ssl_context));
+		}
+#endif
+		session->rproxy_kept_alive = true;
+	}
 
 	return result;
 }
